{
> I would like to include a clock in my current project which will be
> updated once a minute.  Instead of constantly checking the computer's clock
> and waiting for it to change, I would like to use an interrupt.

This one has even a hot key handler.  If you want to update it once per
minute, bump a counter within the interrupt 1Ch handler till it reaches the
value 60*18.2.  Then refresh the screen.
}

Program Clock;

{$G+,R-,S-,M 1024, 0, 0 }

uses
  Dos,Crt;

Const
  x           = 0;                   { x location on screen }
  y           = 1;                    { y location on screen }
  Keyboard    = 9;                    { Hardware keyboard interrupt }
  TimerTick   = $1C;                  { Gets called 18.2 / second }
  MSDOS       = $21;
  VideoOffset = 160 * (y - 1) + 2 * x;{ Offset in display memory }
  yellow      = 14;
  blue        = 1;
  attribute   = blue * 16 + yellow;   { Clock colours }
  VideoBase   : Word = $B800;         { Segment of display memory }
  ActiveFlag  : ShortInt = -1;        { 0: on, -1: off }

Var
  OrgInt9,                             { Saved interrupt 9 vector }
  OrgInt21h,                           { Saved interrupt 21h vector }
  OrgInt1Ch : Pointer;              { Saved interrupt 1Ch vector }
  VideoMode : Byte absolute $0000:$0449;
  fseg,fofs:word;
{ Display a string using Dos services (avoid WriteLn, save memory) }

  r,g,b:byte;
  mag_gaan:word;
Procedure DisplayString(s : String); Assembler;

ASM
  PUSH   DS
  XOR    CX, CX
  LDS    SI, s
  LODSB
  MOV    CL, AL
  JCXZ   @EmptyString
  CLD
 @NextChar:
  LODSB
  XCHG   AX, DX
  MOV    AH, 2
  INT    21h
  LOOP   @NextChar
 @EmptyString:
  POP    DS
end;

{ Returns True if a real time clock could be found }
Function HasRTClock : Boolean; Assembler;

ASM
  XOR    AL, AL
  MOV    AH, 2
  INT    1Ah
  JC     @NoRTClock
  INC    AX
 @NoRTCLock:
end;

{ Release Dos environment }
Procedure ReleaseEnvironment; Assembler;
ASM
  MOV    ES, [PrefixSeg]
  MOV    ES, ES:[002Ch]
  MOV    AH, 49h
  INT    21h
end;

{ INT 9 handler intercepting Alt-F11 }
Procedure ToggleClock; Interrupt; Assembler;
Const
  F11      = $57;                  { 'F11' make code }
  BiosSeg  = $40;                  { Segment of BIOS data area }
  AltMask  = $08;                  { Bitmask of Alt key }
  KbdFlags = $17;                  { Byte showing keyboard status }

ASM
  STI
  IN     AL, 60h

 { F11 pressed? }
  CMP    AL, F11
  JNE    @PassThru

 { Alt-key pressed? }
  PUSH   BiosSeg
  POP    ES
  MOV    AL, ES:[KbdFlags]
  AND    AL, AltMask
  CMP    AL, AltMask
  JNE    @PassThru

 { Flip status flag, force EOI and leave routine }
  NOT    [ActiveFlag]
  IN     AL, 61h
  MOV    AH, AL
  OR     AL, 80h
  OUT    61h, AL
  MOV    AL, AH
  OUT    61h, AL
  CLI
  MOV    AL, 20h
  OUT    20h, AL
  STI
  JMP    @Exit

 @PassThru:
  CLI
  PUSHF
  CALL   DWord Ptr [OrgInt9]
 @Exit:
end;  { ToggleClock }

{ Convert a packed BCD byte to ASCII character }
Procedure Digit; Assembler;
ASM
  PUSH   AX
  CALL   @HiNibble
  POP    AX
  CALL   @LoNibble
  RETN

 @HiNibble:
  SHR    AL, 4
  JMP    @MakeAscii
 @LoNibble:
  AND    AL, 0Fh
 @MakeAscii:
  OR     AL, '0'
  STOSW
end;
procedure setpal(c,r,g,b:byte); assembler; asm
  mov dx,3c8h; mov al,[c]; out dx,al; inc dx; mov al,[r]
  out dx,al; mov al,[g]; out dx,al; mov al,[b]; out dx,al; end;
procedure getfont; assembler; asm
  mov ax,1130h; mov bh,6; int 10h; mov fseg,es; mov fofs,bp; end;
procedure writetxt(x,y:word; txt:string);
var i,j,k:byte;
begin
  for i:=1 to length(txt) do for j:=0 to 15 do for k:=0 to 7 do
    if ((mem[fseg:fofs+ord(txt[i])*16+j] shl k) and 128) <> 0 then begin
      mem[$a000:(y+j)*320+((i-1)*8)+x+k]:=63;
    end;
end;

procedure naar_VGA;
begin
  asm
    Mov AX,93h; {13h}
    Int 10h  { Go_Grafix_Mode 320x200x256}
  end;
  Directvideo:=false;
end;

procedure plasmatic;
var
  cntr,cntr2:word;
  val:byte;
begin
  setpal(0,16,16,16);
  setpal(63,63,63,63);
  for cntr2:=0 to 80 do begin
    for cntr:=0 to 319 do begin
      val:=mem[$a000:cntr+cntr2 shl 6+cntr2 shl 8];
      if (val=0) OR (val>63) then mem[$a000:cntr+cntr2 shl 6+cntr2 shl 8]:=(cntr+cntr2) AND 15+128;
    end;
  end;
end;

{ INT 1Ch handler that displays a clock on the right hand side of the screen }
Procedure DisplayClock; Interrupt;
begin
ASM
  CMP    [ActiveFlag], 0
  JNE    @Exit
  CLD
  MOV    AH, 2
  INT    1Ah
  MOV    ES, [VideoBase]
  MOV    DI, VideoOffset
  MOV    AH, attribute {!}
  MOV    AL, CH
  CALL   Digit
  MOV    AL, ':'
  STOSW
  MOV    AL, CL
  CALL   Digit
  MOV    AL, ':'
  STOSW
  MOV    AL, DH
  CALL   Digit         {!}

  (*
  mov ax,$0f00
  int 10h
{ mov c,ah {kolommen}
{ mov m,al {mode}
{ mov p,bh {Page}
    cmp al,13h
    jne @geen_13h
    call plasmatic
    jmp @geen_textmode
  @geen_13h:
    cmp al,3
    jne @geen_textmode
    call naar_VGA
  @geen_textmode:
*)
  PUSHF
  CALL   DWord Ptr [OrgInt1Ch]
 @Exit:
end;
end;

Procedure MSDOSbypass; Interrupt;
begin
ASM
  JMP DWord Ptr [OrgInt21h]
end;
end;
procedure get_pal;
var
  c,m,p:byte;
begin
  asm
    CMP    [ActiveFlag], 0
    JNE    @Exit
    CLD
    MOV    AH, 2
    INT    1Ah
    mov ax,$0f00
    int 10h
    mov c,ah {kolommen}
    mov m,al {mode}
    mov p,bh {Page}
    MOV    ES, [VideoBase]
    MOV    DI, VideoOffset
    MOV    AH, attribute
    mov    AL, 'M'
    stosw
    cmp al,3
    jne @geen_textmode
    Mov AX,13h;
    Int 10h  { Go_Grafix_Mode 320x200x256}
  @geen_textmode:
    PUSHF
    CALL   DWord Ptr [OrgInt1Ch]
   @Exit:
  end;
end;



{NEW CODE}

procedure MYrout;
begin
{    db $9A;
    dw offset MYrout,seg MYrout;}
end;

procedure insert_MSDOS_bypass;
begin
  asm

    mov di,ds:[offset OrgInt21h]
    mov cs:[offset @l3],di
    mov ax,ds:[offset OrgInt21h+2]
    mov cs:[offset @l3+2],ax
    mov es,ax

    mov al,es:[di]                {SAVE MS-DOS routine}
    mov cs:[offset @l1],al
    mov ax,es:[di+1]
    mov cs:[offset @l1+1],ax
    mov ax,es:[di+3]
    mov cs:[offset @l1+3],ax

    mov al,$EA                    {Overschrijf MS-DOS routine}
    mov es:[di],al
    mov ax,offset @l2 {MYrout}
    mov es:[di+1],ax
    mov ax,seg    @l2 {MYrout}
    mov es:[di+3],ax


    jmp @quit


@l1:dw $0,$0,$0   {OUDE ML van MSDOS ,die ik ga overschrijven}
@l2:
    cli
    {restore MS-DOS code}
    pusha
    mov di,0            {ZET 'H' OP SCHERM}
    mov ax,$B800
    mov es,ax
    mov al,'H'
    mov ah, attribute
    mov es:[di],ax

{
    mov di,ds:[offset OrgInt21h]
    mov ax,ds:[offset OrgInt21h+2]
    mov es,ax

    mov al,cs:[offset @l1]
    mov es:[di],al
    mov ax,cs:[offset @l1+1]
    mov es:[di+1],ax
    mov ax,cs:[offset @l1+3]
    mov es:[di+3],ax
}
    popa

    mov mag_gaan,1

{   pushf}
    {call MS-DOS code}
    db $9A;
@l3:dw $0,$0;   {hier komt de SEG+OFS naar de MS-DOS routine}

    sti
    (*
    {Overschrijf MS-DOS code}
@l4:pusha
    mov di,ds:[offset OrgInt21h]
    mov ax,ds:[offset OrgInt21h+2]
    mov es,ax
    mov al,$EA                    {Overschrijf MS-DOS routine}
    mov es:[di],al
    mov ax,offset @l2 {MYrout}
    mov es:[di+1],ax
    mov ax,seg    @l2 {MYrout}
    mov es:[di+3],ax
    popa
*)
  @quit:
  end;
end;

procedure MS_DOS_fake_routine;
begin
  writeln('HI!');
end;

procedure insert_MSDOS_bypass2;
begin
  asm

    mov di,offset MS_DOS_fake_routine {ds:[offset OrgInt21h]}
    mov ax,seg MS_DOS_fake_routine {ds:[offset OrgInt21h+2]}
    mov es,ax

    mov al,$EA                    {Overschrijf MS-DOS routine}
    mov es:[di],al
    mov ax,offset @l2 {MYrout}
    mov es:[di+1],ax
    mov ax,seg    @l2 {MYrout}
    mov es:[di+3],ax


    jmp @quit


@l2:
    {restore MS-DOS code}
    pusha
    mov di,0            {ZET 'H' OP SCHERM}
    mov ax,$B800
    mov es,ax
    mov al,'H'
    mov ah, attribute
    mov es:[di],ax
    popa

  @quit:
  end;
end;


{END OF NEW CODE}

Begin
  If VideoMode = 7 Then
    VideoBase := $B000;
  getfont;
{ GetIntVec(TimerTick, OrgInt1Ch);
  SetIntVec(TimerTick, @DisplayClock);
  GetIntVec(Keyboard, OrgInt9);
  SetIntVec(Keyboard, @ToggleClock);}
{ GetIntVec(MSDOS   , OrgInt21h);}
{ SetIntVec(MSDOS   , @MSDOSbypass);}

  insert_MSDOS_bypass2;
  MS_DOS_fake_routine;

{ mag_gaan:=0;
{ repeat
  until mag_gaan=1;}

{ SwapVectors;
  ReleaseEnvironment;
  DisplayString('CLOCK installed.  <Alt-F11> toggles on/off');
  Keep(0);}
end.
