Program MIDI_Reader;
Uses Dos, Crt;

{ -------------------------------------------------------------------- }
Type
{MIDI}
   TDataPacket  = record
                     Command    : Byte; { See MIDI-Commands below }
                     Channel    : Byte; { 0..15 }
                     Data_1     : Byte; { Usually: NoteNr. or Controler Nr }
                     Data_2     : Byte; { Value }
                   end;
{ -------------------------------------------------------------------- }
{DMA}
    TDmaBlock = record   { Big samples never stay in a Physical page }
      P1,P2   : Pointer; { So I split them Up, see below how.. }
      S1,S2   : Word;
      Played2 : Boolean;
    end;

{ -------------------------------------------------------------------- }
Const
{MIDI}
     MPU_reset      = $ff;
     MPU_UART_Mode  = $3f;
     MPU_ack        = $fe;
     MPU_Dataport   = $330;
     MPU_Status     = $331;
     MPU_Data_Avail = $80; { When not using irq's, compare statusport with }
     MPU_Data_Ready = $40; { these: Ready_to_write := Port[Status] and $40 }
     MPU_Int        = $0A; { SoftWare Interrupt generated by Hardware IRQ 2}
     MPU_Int_Mask   = $04; { Bit 2 = status for IRQ 2 }
{     MPU_Int        = $0F; { SoftWare Interrupt generated by Hardware IRQ 7}
{     MPU_Int_Mask   = $80; { Bit 7 = status for IRQ 7}

{ For Future Use }
     DSP_Int        = $0D; { SoftWare Interrupt generated by Hardware IRQ 7}
     DSP_Int_Mask   = 16; { Bit 7 = status for IRQ 7 }
                                            { nb. bits  are 0..7 }

{ --- MIDI-Commands --- }
                     { Status:  Data_1:  Data_2:  }
     Note_off = $8;  { 1000nnnn 0kkkkkkk 0vvvvvvv  k = notenr. v = velocity  }
     Note_On  = $9;  { 1001nnnn 0kkkkkkk 0vvvvvvv  v = 0 means also note off }
     KeyPres  = $A;  { 1010nnnn 0kkkkkkk 0vvvvvvv                            }
     Control  = $B;  { 1011nnnn 0ccccccc 0vvvvvvv  c = controler 0..127      }
     Progr    = $C;  { 1100nnnn 0ppppppp           p = Program Change 0..127 }
     Chn_Prs  = $D;  { 1101nnnn 0vvvvvvv                                     }
     Pitch    = $E;  { 1110nnnn 0bbbbbbb 0vvvvvvv  b = LSB v = MSB (0..16383)}
     Syst     = $F;  { 1111nnnn                    like songpos, start, stop }
     Sysex    = $F;  { 11110000                    sysex, burk               }
     {ActiveSense =254 11111110 }

    { One-byte commands are not put into the Stream ! }
    { Please don't send Sysex I haven't implemented it, you'll get garbage }
    { EndSysex = $F7, Sysex = $F0 }

     MaxStream = 24; { maximum of (complete) commands in Stream }
{----------------------------------------------------------------------}
{DMA}
   Mdebug : Boolean = False;
   fname = 'D:\music\samples2\funky.raw';  { Can be any raw data File }
   BASE       = $220;         { Change this to your card base adres and }
   Jumper     = $5;           { IRQ-setting of your SB (compatible)...  }
   SB_IRQ     = Jumper + $8;  { This will be the interrupt generated by SB }
   IRQ_Contr  = $21;          { IRQ controller 1 (IRQ-Jumper 2, 5, 7)
                                (Controller 2 (IRQ-Jumper 10 at Port $A1,
                                 Bit 2 see next item too but then also
                                 haveto enable IRQ 2 at Controller 1) }
   BitMask    = $20 ;          { Oh my god, how I'm gonna explain this one ?
                                Ok, here it is: You gotta enable the irq
                                on the IRQ controller. The Bits in the byte
                                tell wich IRQ to enable. So to enable a
                                hardware irq (jumper setting), set the
                                corresponding bit to Zero. I hope you know
                                how to do that...
                                   Bit: 7  6  5  4  3  2  1  0
                                        ^     ^        ^
                                     IRQ7  IRQ5     IRQ2 and IRQ10

                                 So For IRQ 10 > enable IRQ 2 (set bit 2 to
                                 zero) at Port $21, and enable IRQ 10 (set
                                 bit 2 to zero) at Port $A1. <pffff>
                                 Just look at the INIT procedure...   }
   resetport  = Base + $6;    { the SB_ports used }
   readport   = Base + $A;
   Writeport  = Base + $C;
   statusport = Base + $E;
   HexTable : Array[0..15] of Char =
       ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');

{--  commands to WritePort ------------------------------------------}
   DAC_Write  = $10; { DIRECT 8-BIT DAC (Single Byte Data Transfer) }
   DMA_8BIT   = $14; { DMA MODE 8-BIT DAC }
   ADC_Read   = $20; { DIRECT MODE (Single Byte Data Transfer)}
   speakerON  = $D1;
   speakerOFF = $D3;
   DMACont    = $D4;
   DMAStop    = $D0; { Strange enough, send first to install IRQ somehow }


{ -------------------------------------------------------------------- }
Var
     Old_MPU_intvec : Pointer;
     Old_DSP_intvec : Pointer;
     Old_IRQ_mask   : Byte;
     IRQmask        : Byte;
     NEW_IRQ_mask        : Byte;

     MIDIstream : Array[0..MaxStream] of TDataPacket;
     MIDIpacket : TDataPacket; { Packet used by interrupt-handler }
     Head       : Byte;        { Where are we in the stream }

    Sound     : Pointer;
    Buffer_dma: pointer;
    Size      : Word;
    Playing   : Boolean;
    OldIntVec : Pointer;  { Old IRQ 0Fh Pointer }
{    IRQmaskDMA: Byte;     { Store Interrupt Controler's status. Port $21 }
    DMAblock  : TDMAblock;
{DMA}
  Ch       : char;
  SegS,
  OfsS,
  BLeft,
  B_S2     : Word;
  NextPage : lONGiNT;
{MARK}
  Sb_Irq_Status:byte;
  Midi_Irq_Status:byte;
  DMAsend_Irq_Status:byte;
  Sb_Irq_Meter:byte;
  Midi_Irq_Meter:byte;
  DMAsend_Irq_Meter:byte;
  Adres:Longint;
  PlaybufferLen:Word;
  adres_SEG,adres_OFS:Word;

{ -------------------------------------------------------------------- }

 fUNCtION hEX (tHEvAR:Byte) :sTRING; {See what the hex it looks like }
vAR
 bUF    : sTRING;
 w      : wORD;
bEGIN
 w := tHEvAR;
 bUF:=      hEXtABLE[ (w) sHR  4];
 bUF:=bUF + hEXtABLE[ (w) mOD 16];
 hEX:=bUF;
eND;
{----------------------------------------------------------------------}

Function ResetDSP: Boolean;
var
  TimeDone : LongInt;

Begin
 TimeDone:=0;
 Port[ResetPort]:=1;          { Port $226 }
 Asm Nop;Nop;Nop; End;
 Port[ResetPort]:=0;
 Repeat
   Inc(TimeDone);             { I don't wanna stay here 4 ever }
 Until (TimeDone>123000) or (Port[ReadPort] = $AA);
 ResetDSP:=(TimeDone<123000); { exited with port-read instead by TimeDone }
End;
{----------------------------------------------------------------------}

Procedure WriteDSP(Value: Byte);
Begin
  repeat until (Port[WritePort] And $80)=0; { Until bit 7 is set }
  Port[WritePort] := value;
End;

{---------------------------------------------------------------------}

procedure SendDMA(sound : Pointer; size : word); { Send Sample Via DMA }
var page, offset : Word;
begin
  size := size - 1;

  { Set up the DMA chip }
  Offset := (Seg(sound^) Shl 4) + Ofs(sound^);      { 20-bit physical adres}
  page := (Seg(sound^) + Ofs(sound^) shr 4) shr 12; { Page 0..F }
  Port[$0A] := 5;              { Mask for Channel 1 }
  Port[$0C] := 0;              { Clear Flip-Flop ?? }
  Port[$0B] := $49;            { DMA > DAC mode. (ADC > DMA = $45) }
  Port[$02] := Lo(offset);     { Low Byte Base Offset Adress }
  Port[$02] := Hi(offset);     { High Byte Base Offset Adress }
  Port[$83] := page;           { Physical memory Page number }
  Port[$03] := Lo(size);       { Low Byte DMA Data Count }
  Port[$03] := Hi(size);       { High Byte DMA Data Count }
  Port[$0A] := 1;              { Enable DMA channel 1 }

  { Set the playback type (8-bit) }
  WriteDSP( DMA_8BIT );        { DMA > DAC 8-Bit mode }
  WriteDSP( Lo(size) );        { Low Byte DMA Data Count }
  Playing:=True;
  WriteDSP( Hi(size) );   { DSP Starts at High Byte Data Count recieved }
  DMAsend_Irq_Status:=1;   {passed!}
{  if Mdebug then begin GotoXY(30,3);Writeln('DMA > DAC '); end;}
end;

{---------------------------------------------------------------------}

procedure SendDMAQ(sound : Pointer; size : word); { Send Sample Via DMA }
var page, offset : Word;
begin
  size := size - 1;

  Port[$0A] := 1;              { Mask for Channel 1 }

  { Set the playback type (8-bit) }
  WriteDSP( DMA_8BIT );        { DMA > DAC 8-Bit mode }
  WriteDSP( Lo(size) );        { Low Byte DMA Data Count }
  Playing:=True;
  WriteDSP( Hi(size) );   { DSP Starts at High Byte Data Count recieved }
  DMAsend_Irq_Status:=1;   {passed!}
{  if Mdebug then begin GotoXY(30,3);Writeln('DMA > DAC '); end;}
end;

{---------------------------------------------------------------------}

procedure CalcDMA(sound : Pointer; size : word); { Calc DMA-Sample }
begin
   adres_SEG:=SEG(Sound^);
   adres_OFS:=OFS(Sound^);
   asm
     push ds    { bewaar ds}
     push si
     push ax
     push cx

     push adres_SEG
     pop  ds
     mov  si,adres_OFS
     mov  ax,128
     mov  cx,Size
     shr  cx,1
     shr  cx,1
     shr  cx,1
@clea:
     mov  ds:[si],ax
{     add  ax,257}
     mov  ds:[si]+2,ax
{     add  ax,257}
     mov  ds:[si]+4,ax
{     add  ax,257}
     mov  ds:[si]+6,ax
{     add  ax,257}
     add  si,8
     LOOP @clea

     pop  cx
     pop  ax
     pop  si
     pop  ds
   end;
end;


{----------------------------------------------------------------------}

Procedure SB_intHandler; Interrupt;
var A : Byte;
(*    Ŀ
       IRQ Jumper   Interrupt 
        Setting     Generated 
      Ĵ
          2            0Ah    
          3            0Bh    
          5            0Dh    
          7            0Fh    
      *)
Begin
 A:=Port[$22E]; {Acknowledge Interrupt from SB, to SB, by reading port $22E}

 Port[$3c8]:=0;
 Port[$3c9]:=63;
 Port[$3c9]:=63;
 Port[$3c9]:=63;
 {Proces here something, like playing next block }
 if DMAblock.played2 = False then
 begin
   SendDMAq(DMAblock.P2, DMAblock.S2);
   DMAblock.Played2 := false;
   CalcDMA(DMAblock.P1, DMAblock.S1);
 end else
 begin
   SendDMAq(DMAblock.P1, DMAblock.S1);
   DMAblock.Played2 := False;
   CalcDMA(DMAblock.P2, DMAblock.S2);
 end;

 Port[$3c8]:=0;
 Port[$3c9]:=0;
 Port[$3c9]:=0;
 Port[$3c9]:=0;

 Sb_Irq_Status:=1;   {passed!}
 Port[$20]:=$20; { EOI, End Of Interrupt to Interrupt Controller }
end;


{----------------------------------------------------------------------}





{ -------------------------------------------------------------------- }
Procedure MPU_INT_HANDLER; Interrupt;  { A byte is waiting at MPU, get it }
Var I         : Integer;
    Mbyte     : Byte;
    Stream_It : Boolean;
Begin
 Asm Cli end;
 { I had no problems leaving this out: Stream_it:=False;}

   Mbyte := Port[MPU_Dataport];
   If (MByte and 128) = 128 then {a command byte is send }
     Begin
       MIDIpacket.Command := Mbyte shr 4; { for comparing command constants }
       MIDIpacket.Channel := Mbyte mod 16;{ The channel nr. 0..15           }
       MIDIpacket.Data_1  := 0;           { Initialize for next reads }
       MIDIpacket.Data_2  := 0;
     end else {it's a DataByte}
     begin
       if MIDIpacket.Data_1 = 0 then      { the first data byte }
          begin
            MIDIpacket.Data_1 := Mbyte;
            Stream_it := ((MIDIpacket.Command = Progr)or
                          (MIDIpacket.Command = Chn_Prs));{have 1 data byte}
          end else
          begin                            { second data byte and then}
            MIDIpacket.Data_2 := Mbyte;
            Stream_it := True;             { put it in the stream }
          end;

       if Stream_it then
          begin
            inc(head); { We have to increase stream position and maybe, }
            if Head > Maxstream then {loose data and }
                 begin
                   For I := 1 to 24 do MIDIstream[I-1]:=MIDIstream[I];
                   Head  := MaxStream; { stay in bounderies }
                 end;
            MIDIstream[Head]   := MIDIpacket; { put Packet in the stream }
            MIDIpacket.Data_1  := 0;
            { this needs explanation : On my yamaha keyboard it turned out  }
            { that if notes were played fast (a chord is send as 3 seperate }
            { notes, very fast) then only one status byte comes in,         }
            { followed by more notenumber-velocity-pairs (data bytes).      }
            { This means that the same statusbyte is to be used until a     }
            { new one is read. So here we set Data_1 to 0 so the next byte  }
            { (if it's not a statusbyte) will be read as a Data_1 byte.     }
          end;
     end;

   Port[MPU_status]:=MPU_ack; { AckNowledge Interrupt to MPU }
 Asm Sti end;
 Midi_Irq_Status:=1;   {passed!}
 Port[$20] := $20 {EOI, End-Of-Interrupt to Interrupt Controller, I'm ready}
end;

{----------------------------------------------------------------------}

Procedure SetFreq(frequency: Word); { Set DMA > DSP playback frequency }
Var TIME_CONSTANT : word;
Begin
  TIME_CONSTANT := 256 - 1000000 div frequency; { From da bookZ }
  WriteDSP($40);
  WriteDSP(TIME_CONSTANT);
end;

{ -------------------------------------------------------------------- }
Procedure INIT;
Begin
{general}
 Old_IRQ_mask := Port[$21];{So IRQ controller, this is what you got enabled}

 IRQmask:=Old_IRQ_mask;


{MIDI}
 Head:= 0;
 GetIntVec(MPU_int , Old_MPU_Intvec);   { Why are you pointing there ? }
 SetIntVec(MPU_int , @MPU_INT_HANDLER); { Point to this man          ! }
 Port[MPU_status]:=MPU_reset;           { Hey man, are U still sleeping ? }
 Port[MPU_status]:=MPU_UART_mode;       { Gimme some connection man     ! }
{ If (IRQmask and MPU_int_mask) = MPU_int_mask then { Hey, IRQ controller... }

 IRQmask :=(IRQmask Xor MPU_int_mask);{Enable this IRQ I want to read }

{DMA}
 ResetDSP;
 Asm Cli End;
{   If IRQmask and BitMask = Bitmask then}
      if (Port[IRQ_Contr] and 32 )= 32 then Port[IRQ_Contr]:=Port[IRQ_Contr] xor 32; { Set bit to 0 to enable IRQ }
   PlayIng := False;
 Asm Sti End;
 GetIntVec(SB_IRQ, OldIntVec);
 SetIntVec(SB_IRQ, @SB_IntHandler);
 WriteDSP(SpeakerON);
 SetFreq(16000);
end;

{ -------------------------------------------------------------------- }
Procedure DONE;
Begin
 {general}
 Port[$21]:= Old_IRQ_Mask; { OK, I'll restore IRQ status }
 WriteDSP(SpeakerOFF);
 WriteDSP(DMAstop);
 Asm Cli End;
 {DMA}
   SetIntVec(SB_IRQ, OldIntVec);
 {MIDI}
   SetIntVec(MPU_int,Old_MPU_Intvec);
 Asm Sti End;
end;

{---------------------- LoadRawSampleData --------------------}

Function loadFile(Var buffer:Pointer; Filename:String) : Word;
Var
   fromf   : File;
   size    : Word;
   errcode : Integer;

begin
   assign(fromf,Filename);
   reset(fromf,1);
   errcode := ioresult;
   if errcode = 0 then
    begin
      Size := Filesize(fromf);
      if Size > 65527 then Size := 65527; { not bigger than 64k - 9 bytes }
      getmem(buffer,size);
      blockread(fromf,buffer^,size);
    end
   else size := 0;
   loadFile := size;
   close(fromf);
end;

{----------------------------------------------------------------------}

fUNCtION hEX_iNT (tHEvAR:lONGiNT) :sTRING; {See what it looks like in HEX }
vAR
 bUF      : sTRING;
 wLO,wHI  : wORD;
bEGIN
 wLO := tHEvAR SHR 15;
 wHI := tHEvAR MOD 65535;
 bUF:=      hEXtABLE[ hI(wHI) sHR  4];
 bUF:=bUF + hEXtABLE[ hI(wHI) mOD 16];
 bUF:=bUF + hEXtABLE[ lO(wHI) sHR  4];
 bUF:=bUF + hEXtABLE[ lO(wHI) mOD 16];;
 bUF:=bUF + hEXtABLE[ hI(wLO) sHR  4];
 bUF:=bUF + hEXtABLE[ hI(wLO) mOD 16];
 bUF:=bUF + hEXtABLE[ lO(wLO) sHR  4];
 bUF:=bUF + hEXtABLE[ lO(wLO) mOD 16];;
 hEX_iNT:=bUF;
eND;


{ -------------------------------------------------------------------- }
Procedure InfoDMA;
begin
  size:= LoadFile(Sound,fname); { Load the DATA }

  WriteLn('Sound Ptr: ',Hex(Seg(Sound^)),':',Hex(Ofs(Sound^)) );
  WriteLn('Size     : ',Hex(size));
  WriteLn(IRQmask);

   { Bytes Left 'till Phys. page boundery, usually starts in $0000 page.
     Should this not be so then change the $10000 }
  SegS:=Seg(Sound^);
  OfsS:=Ofs(Sound^);


    NextPage := (((SegS + OfsS shr 4) shr 12) +1); { Page 0..F }
    Bleft := NextPage - ((SegS * 16) + OfsS);
    B_S2  := Size - Bleft;

With DMAblock do
begin
  If B_S2 > 0 then      { Divide page breaking Var in two }
   Begin
    P1 := Sound;
    S1 := Bleft-2; { the last would be a page further already, he, he. :) }
    P1 := Ptr(NextPage SHL 12, 0);  {zorg dat de lengte v/h DMA-buffer}
    S1 := 800 {B_S2};                     { niet kleiner wordt dan 1 of zo.. MARK}
    P2 := Ptr(NextPage SHL 12, 0);
    S2 := 800 {B_S2};
{    P2 := Nil;
    S2 := 0;}
   end else             { it is not page breaking }
   begin
    P1 := Sound;
    S1 := 800 {Bleft-2}; { the last would be a page further already, he, he. :) }
    P2 := Sound;
    S2 := 800 {Bleft-2}; { the last would be a page further already, he, he. :) }
{    P2 := Nil;
    S2 := 0;}
   end;

  PlaybufferLen:=200;
  Adres:=SEG(buffer_DMA^)*16+OFS(buffer_DMA^);
  if (Adres DIV 65536)=((Adres+PlaybufferLen) DIV 65536) then
  begin
    P1:=Ptr(Adres div 16,Adres and 15);
    S1:=PlaybufferLen;
  end else
  begin
    inc(Adres,PlaybufferLen);
    P1:=Ptr(Adres div 16,Adres and 15);
    S1:=PlaybufferLen;
  end;
  inc(Adres,PlaybufferLen);
  if (Adres DIV 65536)=((Adres+PlaybufferLen) DIV 65536) then
  begin
    P2:=Ptr(Adres div 16,Adres and 15);
    S2:=PlaybufferLen;
  end else
  begin
    inc(Adres,PlaybufferLen);
    P2:=Ptr(Adres div 16,Adres and 15);
    S2:=PlaybufferLen;
  end;

  WriteLn('P1 : ',Hex(Seg(P1^)),':',Hex(Ofs(P1^)) );
  WriteLn('S1 : ',Hex(S1));
  WriteLn('P2 : ',Hex(Seg(P2^)),':',Hex(Ofs(P2^)) );
  WriteLn('S2 : ',Hex(S2));
  WriteLn('S1 + S2 (Should be same as Size): ',Hex( S1 + S2),' (',((S1+S2)/1024):2:2,' Kb)');
  WriteLn('   --- Next Phys. Page (((SegS + OfsS shr 4) shr 12) +1) at: ',Hex_Int(NextPage));
  WriteLn;
  GotoXY(1,20);
  WriteLn('SpaceBar Plays Sample.');
  WriteLn('Q - SetS SampleRate at 16000 Herz');
  WriteLn('W - Sets SampleRate at 22000 Herz');
  WriteLn('Enter = Stop Playing,  Esc = Quit');
end;
end;
{ -------------------------------------------------------------------- }
Procedure ReadPacket(Var Packet: TDataPacket);
var I : Integer;
begin
Asm Cli end;        { gimme change to Read current packet }
 Packet:= MIDIstream[0];
 If Head > 0 then   { hey man, is your program that slow, it needs a stream }
 begin
   Dec(Head);
   For I := 0 to Head do MIDIstream[I]:=MIDIstream[I+1];
   I:=Head+1;
   With MIDIstream[I] do
    Begin
     Command := 0;
     Channel := 0;
     Data_1  := 0;
     Data_2  := 0;
    end;
 end;
Asm Sti end; { got packet, you may interrupt me or whatever }
end;



{ -------------------------------------------------------------------- }

Var
 Troep : TDataPacket;
 I     : Integer;
 x : longint;
Begin
 x:=maxavail;
 ClrScr;
 INIT;
 INFODMA;
 getmem(buffer_dma,30000);
 CalcDMA(DMAblock.P1, DMAblock.S1);   CalcDMA(DMAblock.P2, DMAblock.S2);

 Repeat
  For I:= 1 To 24 do
  begin
   GotoXY(1,i);
   Writeln(Hex(MIDIstream[I].Command),' ',
           Hex(MIDIstream[I].Channel),' ',
           Hex(MIDIstream[I].Data_1),' ',
           Hex(MIDIstream[I].Data_2) );
  end;
  If Sb_Irq_Status=1 then Sb_Irq_Meter:=10;
  If Midi_Irq_Status=1 then Midi_Irq_Meter:=10;
  If DMASend_Irq_Status=1 then DMASend_Irq_Meter:=10;

  gotoXY(30,1);Writeln('SB_IRQ  : ');
  for i:=0 to Sb_Irq_Meter do
  begin gotoXY(40+i,1);Writeln('*');end;
  gotoXY(40+Sb_Irq_Meter+1,1);Writeln('.');
  If Sb_Irq_Meter<>0 then Dec(Sb_Irq_Meter);

  gotoXY(30,2);Writeln('MIDI_IRQ: ');
  for i:=0 to Midi_Irq_Meter do
  begin gotoXY(40+i,2);Writeln('*');end;
  gotoXY(40+Midi_Irq_Meter+1,2);Writeln('.');
  If Midi_Irq_Meter<>0 then Dec(Midi_Irq_Meter);

  gotoXY(30,3);Writeln('DMASend : ');
  for i:=0 to DMASend_Irq_Meter do
  begin gotoXY(40+i,3);Writeln('*');end;
  gotoXY(40+DMASend_Irq_Meter+1,3);Writeln('.');
  If DMASend_Irq_Meter<>0 then Dec(DMASend_Irq_Meter);

  Sb_Irq_Status:=0;     {Clear statussen}
  Midi_Irq_Status:=0;
  DMAsend_Irq_Status:=0;


{  if Port[$60]= 57 then ReadPacket(Troep); { Flush some stream garbage }

    if Keypressed then Ch := Readkey else Ch:=#255;
    Case Upcase(CH) of
    #13: WriteDSP(DMAstop);

    'Q': Begin
          WriteDSP(DMAstop);                 { he he   }
          SetFreq(11500);                    { Slow    }
          DMAblock.Played2 := False;         { Re-Init }
          SendDMA(DMAblock.P1, DMAblock.S1); { Blast it out }
         end;

    'W': Begin
          WriteDSP(DMAstop);
          SetFreq(23000);
          DMAblock.Played2 := False;
          SendDMA(DMAblock.P1, DMAblock.S1);
         end;

    ' ': Begin
           if not playing then   { Play the whole sample }
             begin
               DMAblock.Played2 := False;
               SendDMA(DMAblock.P1, DMAblock.S1);
             end else            { Stop Playing and start over }
             begin
              WriteDSP(DMAstop);
              DMAblock.Played2 := False;
              SendDMA(DMAblock.P1, DMAblock.S1);
             end;
         end; { Case CH of ' ' }
    end; { Case }
 until Ch=#27; { Hope U don't have a KGB-keyboard. (doesn't have ESCape key) }
{DMA}
  FreeMem(BUFFER_DMA,30000); { Why should I, Runtime Lib does it, but OK }
  FreeMem(Sound,Size); { Why should I, Runtime Lib does it, but OK }
 {I'm} DONE;
 ClrScr;
end.





















{----------------------------------------------------------------------}
{----------------------------------------------------------------------
  Description: Play RAW Sound on SB (Compatible) Via DMA
               "Take and Give" - WareZ. & (C)opy-ed Ware
               Tnx go to.. - SwaG
                           - PCGPE (VLA, SBDSP.TXT)
                           - And All Echo's conferencing PASCAL

               Januari 1995     Gerhard Dalenoort

 ----------------------------------------------------------------------}

{------- LOGO Cursor  -----------------------------------------------}

Procedure logo;
Const
  MouseCurSor : Array[0..31] of Word =
    (4095,34815,58079,62031,62031,62031,62031,64079,65351,49632,48880,25471,
     20351,25471,48895,49663,61440,30720,7456,3504,3504,3504,3504,1456,184,
     15903,16655,40064,45184,40064,16640,15872);
Var
  Regs       : registers;
begin
  Regs.AX := 9;
  Regs.BX := 0;    { X of the hot spot }
  Regs.CX := 0;    { Y of the hot spot }
  Regs.DX := Ofs(MouseCurSor);
  Regs.ES := Seg(MouseCurSor);
  Intr($33, Regs);
  Regs.AX := 1;     { ShowMouse }
  Intr($33,Regs);
  Regs.ax:=4;       { SetMouse at CX,DX. (X,Y) }
  Regs.cx:=640 div 2;
  Regs.dx:=480 div 2;
   intr($33,Regs);
end;

