uses Crt, Dos;
const
  Blasters : array[1..5] of String = ('SoundBlaster 1.x',
                                      'SoundBlaster 2.0',
                                      'SoundBlaster Pro',
                                      'SoundBlaster 16 ',
                                      'SoundBlaster AWE');
  IRQIntNums : array[0..15] of Byte =
    ($08, $09, $0A, $0B, $0C, $0D, $0E, $0F,
     $70, $71, $72, $73, $74, $75, $76, $77);
 {DSP Commands}
  DirectDAC         = $10;
  NormalDMADAC      = $14;
  AutoInitDMADAC    = $1C;
  DirectADC         = $20;
  NormalDMAADC      = $24;
  SetTimeConst      = $40;
  SetBlockSize      = $48;
  HighSpeedDMADAC   = $90;
  HighSpeedDMAADC   = $98;
  HaltDMA           = $D0;
  SpeakerOn         = $D1;
  SpeakerOff        = $D3;
  GetSpeakerState   = $D8;
  ContinueDMA       = $D4;
  ExitAInit         = $DA;
  GetVersion        = $E1;

  RegSetStereo      = $0E;

  DMAChannels : array[0..3,0..2] of Byte = (
                  ($87,0,1),
                  ($83,2,3),
                  ($81,4,5),
                  ($82,6,7));
  xMono     = 0;
  xStereo   = 1;

var

  PICPort,            {IRQ stuff}
  IRQStartMask,
  IRQStopMask,
  IRQIntVector : Word;

  DMAStartMask,       {Values for DMA}
  DMAStopMask,
  DMAModeReg   : Byte;

  OldIntVector : Pointer;

  BasePort,          {SB ports}
  MixPort,
  ResetPort,
  ReadPort,
  WritePort,
  DataAvail    : Word;

  IRQ,
  DMAChannel   : Byte;

  SRByte,
  CurBlock     : Byte;

  MaxFreqM,
  MaxFreqS     : Word;   {Maximum mixing rate and mixing type}
  MaxType      : Byte;
  DMABuffers : array[0..1] of Pointer;
  DMABuffer : Pointer;
  Stereo    : Boolean;
  MixFreq: Word;
  Major,Minor : Byte;
  OldExitSB : Pointer;
  i : integer;

procedure ExitSB;far;forward;


procedure WriteDSP(Value: Byte);
  inline
    (
     $8B/$16/>WritePort/    {mov dx, WritePort (Variable) }
     $EC/                   {in  al,dx                    }
     $24/$80/               {and al,80h                   }
     $75/$FB/               {jnz -05                      }
     $58/                   {pop ax                       }
     $8B/$16/>WritePort/    {mov dx,WritePort (Variable)  }
     $EE                    {out dx,al                    }
    );

function ReadDSP: Byte;
  inline
    (
     $8B/$16/>DataAvail/    {mov al, DataAvail  (Variable) }
     $EC/                   {in al,dx                      }
     $24/$80/               {and al,80h                    }
     $74/$FB/               {jz -05                        }
     $8B/$16/>ReadPort/     {mov dx, ReadPort  (Variable)  }
     $EC                    {in al,dx                      }
    );

procedure WaitSB;assembler;
asm
  mov dx,WritePort
 @Wait:
  in al,dx
  test al,128
  jnz @Wait
end;

procedure WriteMixReg(Reg, Value : Byte);assembler;
asm
  mov al,Reg
  mov dx,MixPort
  out dx,al
  inc dx
  in al,dx
  mov al,Value
  out dx,al
end;

function ReadMixReg(Reg : Byte) : Byte;assembler;
asm
  mov al,Reg
  mov dx,MixPort
  out dx,al
  inc dx
  in al,dx
  xor ah,ah
end;

function ResetDSP : Boolean;assembler;
asm
  mov dx,[ResetPort]
  mov al,1
  out dx,al

  mov cx,65535
  rep cld        {Wait a while (= do something useless}

  xor al,al
  out dx,al
  mov dx,[DataAvail]

  xor cx,cx
 @Wait:
  in al,dx
  dec cx
  jz @Error
  test al,7
  jz @Wait

  mov dx,[ReadPort]
  in al,dx

  cmp al,$AA
  jne @Error
  mov ax,True
  jmp @End

 @Error:
  mov ax,False
 @End:
end;

procedure TurnSpeakerOn;
begin
  WriteDSP(SpeakerOn);
end;

procedure TurnSpeakerOff;
begin
  WriteDSP(SpeakerOff);
end;

procedure SetStereo;
begin
  WriteMixReg(RegSetStereo, ReadMixReg(RegSetStereo) or 2);
end;

procedure SetMono;
begin
  WriteMixReg(RegSetStereo, ReadMixReg(RegSetStereo) and 253);
end;

procedure GetSBVersion(var Major, Minor : Byte);
begin
  WriteDSP(GetVersion);
  Major:=ReadDSP;
  Minor:=ReadDSP;
end;

procedure ValidFreq(var Freq : Word);
begin
  if Stereo then
  begin
    if Freq>MaxFreqS then
      Freq:=MaxFreqS;
  end
  else
  begin
    if Freq>MaxFreqM then
      Freq:=MaxFreqM;
  end;
  Freq:=(1000000 div (1000000 div Freq));
end;

procedure SetFreq(Freq : Word);
var SR : Byte;
begin
  ValidFreq(Freq);
  if Stereo then
    SR:=256-1000000 div (Longint(Freq)*2)
  else
    SR:=256-1000000 div Freq;
  WriteDSP(SetTimeConst);
  WriteDSP(SR);
end;

procedure GetDMAStuff(var Page : Byte; var Offset : Word; BufSize : Word);
{Calculate DMA page and offset}
var
  Dummy     : longint;
begin
  Dummy:=Longint(Seg(DMABuffers[0]^)) shl 4+
          Longint(Ofs(DMABuffers[0]^));
  Page:=Dummy shr 16;
  Offset:=Dummy mod 65536;
  if Offset+ BufSize < Offset then
  begin
    WriteLn('Standard DMA-buffer on page break, using second DMA-buffer');
    Dummy:=Longint(Seg(DMABuffers[1]^)) shl 4+
            Longint(Ofs(DMABuffers[1]^));
    Page:=Dummy shr 16;
    Offset:=Dummy mod 65536;
    DMABuffer:=DMABuffers[1];
  end;
end;

procedure StartSBDMA(BufSize : Word);
{Start DMA transfer}
var
  DMAPage   : Byte;
  DMAOffset : Word;
  DMASize   : Word;
begin
  DMASize:=BufSize div 2;
  if Stereo then DMASize:=DMASize shl 1;
  GetDMAStuff(DMAPage, DMAOffset, BufSize);

  Port[$0A]:=DMAStopMask;     {Stop DMA Transfer}
  Port[$0C]:=$00;             {Clear flip-flop}
  Port[$0B]:=DMAModeReg;
  Port[DMAChannels[DMAChannel,1]]:=Lo(DMAOffset);   {Send data to DMA}
  Port[DMAChannels[DMAChannel,1]]:=Hi(DMAOffset);
  Port[DMAChannels[DMAChannel,2]]:=Lo(DMASize*2-1);
  Port[DMAChannels[DMAChannel,2]]:=Hi(DMASize*2-1);
  Port[DMAChannels[DMAChannel,0]]:=DMAPage;
  Port[$0A]:=DMAStartMask;

  WriteDSP(SetBlockSize);
  WriteDSP(Lo(DMASize-1));
  WriteDSP(Hi(DMASize-1));

  if MixFreq>22000 then
    WriteDSP(HighSpeedDMADAC)
  else
    WriteDSP(AutoInitDMADAC);

end;

{$F+}
procedure SBIntHandler;interrupt;assembler;
asm
  { ...}
end;
{$F-}

procedure StopSBDMA;
begin
  WriteDSP(HaltDMA);          {Stop SB DMA}
  WriteDSP(ExitAInit);        {Stop auto init}
  WriteDSP(HaltDMA);          {Stop SB DMA}
  Port[$0A]:=DMAStopMask;     {Stop DMA Transfer}
end;

procedure InstallIRQ(IRQ : Byte;NewVec : Pointer;var OldVec : Pointer);
begin
  if IRQ <= 7 then
    PICPort := $21   {INTC1}
  else
    PICPort := $A1;  {INTC2}
  IRQIntVector := IRQIntNums[IRQ];
  IRQStopMask  := 1 shl (IRQ mod 8);
  IRQStartMask := not(IRQStopMask);

  Port[PICPort]:=Port[PICPort] or IRQStopMask; {Stop SB IRQ}
  GetIntVec(IRQIntVector, OldVec);
  SetIntVec(IRQIntVector, NewVec);
  Port[PICPort]:=Port[PICPort] and IRQStartMask; {Start SB IRQ}
end;

procedure UnInstallIRQ(IRQ : Byte;OldVec : Pointer);
begin
  if IRQ <= 7 then
    PICPort := $21   {INTC1}
  else
    PICPort := $A1;  {INTC2}
  IRQIntVector := IRQIntNums[IRQ];
  IRQStopMask  := 1 shl (IRQ mod 8);
  IRQStartMask := not(IRQStopMask);

  Port[PICPort]:=Port[PICPort] or IRQStopMask; {Stop SB IRQ}
  SetIntVec(IRQIntVector, OldVec);
end;

function FindDSPPort : Boolean;
var
  DAP,
  Count   : Longint;
  i       : Integer;
begin
  FindDSPPort:=False;
  for i:=0 to 4 do
  begin
    BasePort:=$210+i*16;
    ResetPort:=BasePort+$6;
    ReadPort:=BasePort+$A;
    DataAvail:=BasePort+$E;

    Write('Trying port ',BasePort,'...   ');
    Port[ResetPort]:=1;
    Delay(1);
    Port[ResetPort]:=0;
    Count:=0;
    repeat
      Inc(Count);
    until ((Port[DataAvail] and 128)=128) or (Count=10000);
    DAP:=Port[ReadPort];
    if (DAP=$AA) and (Count<10000) then
    begin
      FindDSPPort:=True;
      WriteLn('Found a SB.');
      Exit;
    end
    else
      WriteLn('Failed.');
  end;
end;

{$F+}
procedure IRQ2;interrupt;
var X : Byte;
begin
  IRQ:=2;
  X:=Port[DataAvail];
  Port[$20]:=$20;
end;

procedure IRQ3;interrupt;
var X : Byte;
begin
  IRQ:=3;
  X:=Port[DataAvail];
  Port[$20]:=$20;
end;

procedure IRQ5;interrupt;
var X : Byte;
begin
  IRQ:=5;
  X:=Port[DataAvail];
  Port[$20]:=$20;
end;

procedure IRQ7;interrupt;
var X : Byte;
begin
  IRQ:=7;
  X:=Port[DataAvail];
  Port[$20]:=$20;
end;
{$F-}

function TryDMA(Channel : Byte) : Boolean;
var
  DMAStartMask,
  DMAStopMask,
  DMAModeReg    : Word;
begin
  DMAStartMask:=Channel + $00;
  DMAStopMask :=Channel + $04;
  DMAModeReg  :=Channel + $48;

  Port[$0A]:=DMAStopMask;     {Stop DMA Transfer}
  Port[$0C]:=$00;             {Clear flip-flop}
  Port[$0B]:=DMAModeReg;
  Port[DMAChannels[Channel,1]]:=0;
  Port[DMAChannels[Channel,1]]:=0;
  Port[DMAChannels[Channel,2]]:=0;
  Port[DMAChannels[Channel,2]]:=0;
  Port[DMAChannels[Channel,0]]:=0;
  Port[$0A]:=DMAStartMask;

  WriteDSP(SetTimeConst);
  WriteDSP(256-1000000 div 5000);
  WriteDSP(NormalDMADAC);
  WriteDSP(0);
  WriteDSP(0);

  Delay(100);

  if IRQ<>0 then
    TryDMA:=True
  else
    TryDMA:=False;
end;

function FindDMAIRQ : Boolean;
const
  DMAs    : array[0..2] of Byte = (0,1,3);
var
  OldVecs : array[0..3] of Pointer;
  i       : Integer;
begin
  DMAChannel:=255;
  InstallIRQ(2,@IRQ2,OldVecs[0]);
  InstallIRQ(3,@IRQ3,OldVecs[1]);
  InstallIRQ(5,@IRQ5,OldVecs[2]);
  InstallIRQ(7,@IRQ7,OldVecs[3]);

  TurnSpeakerOff;

  for i:=0 to 2 do
  begin
    Write('Trying DMA channel ',DMAs[i],'...   ');
    if TryDMA(DMAs[i]) then
    begin
      DMAChannel:=DMAs[i];
      WriteLn('Succesfull with IRQ ',IRQ);
      Break;
    end
    else
      WriteLn('Failed.');
  end;

  UnInstallIRQ(2,OldVecs[0]);
  UnInstallIRQ(3,OldVecs[1]);
  UnInstallIRQ(5,OldVecs[2]);
  UnInstallIRQ(7,OldVecs[3]);

  if DMAChannel=255 then
    FindDMAIRQ:=False
  else
    FindDMAIRQ:=True;
end;

function DetectSB : Boolean;
begin
  WriteLn('Auto-detecting SoundBlaster');
  if FindDSPPort=False then
  begin
    DetectSB:=False;
    Exit;
  end;
  MixPort   := BasePort + $4;
  ResetPort := BasePort + $6;
  ReadPort  := BasePort + $A;
  WritePort := BasePort + $C;
  DataAvail := BasePort + $E;

  WriteDSP(SetTimeConst);
  WriteDSP(256-1000000 div 5000);
  DetectSB:=FindDMAIRQ;

  GetSBVersion(Major, Minor);
  if Major=1 then
    MaxFreqM:=22222
  else
    MaxFreqM:=45454;
  if Major>2 then
  begin
    MaxFreqS:=45454;
    MaxType:=xStereo;
  end
  else
    MaxType:=xMono;

  Write('Located a ');
  if (Major=4) and (Minor>=12) then
    WriteLn(Blasters[Major+1])
  else
    WriteLn(Blasters[Major]);

end;

function ParseBlaster : Boolean;
var T, i,
    A, B, C,
    Code    : Integer;
    Dummy,
    Dat     : String;

begin
  for T:=1 to EnvCount do
  begin
    Dummy:=EnvStr(T);
    if Pos('BLASTER', Dummy)=1 then
      Break;
  end;
  if Pos('BLASTER',EnvStr(T)) <> 1 then
  begin
    ParseBlaster:=False;
    Exit;
  end;
  i:=Pos('=',EnvStr(T))+1;
  Dummy:=Copy(EnvStr(T),i,Length(EnvStr(T))-i);
  i:=Pos('A', Dummy);
  if i=0 then
  begin
    ParseBlaster:=False;
    Exit;
  end;
  Inc(i);
  Dat:=Copy(Dummy,i,3);
  Val(Dat[1],A,Code);
  if A<>2 then
  begin
    ParseBlaster:=False;
    Exit;
  end;

  Val(Dat[2],B,Code);
  Val(Dat[3],C,Code);
  if C<>0 then
  begin
    ParseBlaster:=False;
    Exit;
  end;
  BasePort:=A shl 8 + B shl 4 + C;

  i:=Pos('D', Dummy);
  if i=0 then
  begin
    ParseBlaster:=False;
    Exit;
  end;
  Inc(i);
  Dat:=Copy(Dummy,i,1);
  Val(Dat[1],A,Code);
  DMAChannel:=A;

  i:=Pos('i', Dummy);
  if i=0 then
  begin
    ParseBlaster:=False;
    Exit;
  end;
  Inc(i);
  Dat:=Copy(Dummy,i,1);
  Val(Dat[1],A,Code);
  IRQ:=A;
end;

function InitSB(IRQ: Byte; BaseIO: Word; DMAChannel: Byte): boolean;
var
  Success      : Boolean;
begin
  if IRQ <= 7 then
    PICPort := $21   {INTC1}
  else
    PICPort := $A1;  {INTC2}
  IRQIntVector := IRQIntNums[IRQ];
  IRQStopMask  := 1 shl (IRQ mod 8);
  IRQStartMask := not(IRQStopMask);

  MixPort   := BaseIO + $4;
  ResetPort := BaseIO + $6;
  ReadPort  := BaseIO + $A;
  WritePort := BaseIO + $C;
  DataAvail := BaseIO + $E;

  DMAStartMask := DMAChannel + $00; {000000xx}
  DMAStopMask  := DMAChannel + $04; {000001xx}
  DMAModeReg   := DMAChannel + $58; {010110xx} {Single mode, autoinit, read}

  Success := ResetDSP;

  InitSB := Success;

{  WriteMixReg($22,(Volume shr 3) shl 5 + (Volume shr 3) shl 1 + 16 + 1);
  WriteMixReg($04,255);
}
end;

procedure StartPlaying(BufSize : Word);
var Dummy : Longint;
    D     : Integer;

begin
  if not InitSB(IRQ, BasePort, DMAChannel) then
  begin
{Error}
  end;
  SetFreq(MixFreq);
  if Stereo then
    SetStereo
  else
    SetMono;
  TurnSpeakerOn;
  InstallIRQ(IRQ, @SBIntHandler, OldIntVector);
  StartSBDMA(BufSize);

  OldExitSB:=ExitProc;
  ExitProc:=@ExitSB;

end;

procedure ExitSB;
begin
  ExitProc:=OldExitSB;
  TurnSpeakerOff;
  StopSBDMA;
  ResetDSP;
  UnInstallIRQ(IRQ, OldIntVector);
end;

begin
  GetMem(DMABuffers[0],1024);
  GetMem(DMABuffers[1],1024);
  for i:=0 to 511 do
  begin
    Mem[Seg(DMABuffers[0]^):Ofs(DMABuffers[0]^)+i*2  ]:=trunc(sin(i/16*2*pi)*63+127);
    Mem[Seg(DMABuffers[0]^):Ofs(DMABuffers[0]^)+i*2+1]:=trunc(sin(i/16*2*pi)*63+127);
    Mem[Seg(DMABuffers[1]^):Ofs(DMABuffers[1]^)+i*2  ]:=trunc(sin(i/16*2*pi)*63+127);
    Mem[Seg(DMABuffers[1]^):Ofs(DMABuffers[1]^)+i*2+1]:=trunc(sin(i/16*2*pi)*63+127);
  end;
  DMABuffer:=DMABuffers[0];
  MixFreq:=44100;
  if not DetectSB then
    Halt;
    stereo:=false;
  BasePort:=$220;
  IRQ:=5;
  DMAChannel:=1;
  MaxFreqS:=44100;
  MaxFreqM:=44100;
  StartPlaying(512);{was 1024}
  delay(100);
  ExitSB;
end.
